
import SOMA.naming.*;
import SOMA.agent.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.lang.reflect.*;

/**
 * Description:  (Agente) Server per il recupero dei risultati
 * @author       Virgilio Pierini
 */

public class ResultManager extends Agent {

  /** Tabella dei risultati */
  Hashtable risultati=new Hashtable();

  /** Socket per la ricezione delle richieste di risultati */
  ServerSocket ss=null;

  /** Porta di ascolto del server */
  final int ServerPort=8000;

  public void run() {
   java.io.PrintStream o=agentSystem.getOut();

   o.println("RESULT MANAGER: preparazione della tabella dei risultati...");
   agentSystem.sharedObjects.put("Risultati",risultati);
   o.println("RESULT MANAGER: tabella pronta.");

   o.println("RESULT MANAGER: preparazione del server...");
   try {
    ss=new ServerSocket(ServerPort);
    while (true) {
     o.println("RESULT MANAGER: in attesa di una richiesta...");
     new ExecuteRequestResultManager(ss.accept(),this).start();
     o.println("RESULT MANAGER: thread avviato.");
    }//while
   } catch (Exception e) { e.printStackTrace(); }
  }

}


/**
 * Description: Classe per la gestione di una singola richiesta concorrente
 */

class ExecuteRequestResultManager extends Thread {
  Socket s;
  BufferedReader sin;
  PrintStream sout;
  PrintStream o;
  Agent agente;

  public ExecuteRequestResultManager(Socket s, Agent agente) {
   this.s=s;
   this.agente=agente;
   this.o=agente.agentSystem.getOut();

   try {
    sin=new BufferedReader(new InputStreamReader(s.getInputStream()));
    sout=new PrintStream(s.getOutputStream());
   } catch (IOException e) {
      try {s.close();} catch (IOException e1) {};
      o.println("Impossibile creare flussi di input e output dalla socket! "+e);
      return; }
  }

  public void run() {
   try {
     o.println("RESULT MANAGER: connessione...");
     String username=sin.readLine();
     sout.println("OK"); //da estendere in futuro per l'autorizzazione
     o.println("RESULT MANAGER: username = "+username);
     String richiesta=sin.readLine();
     o.println("RESULT MANAGER: request = "+richiesta);

     if (richiesta.equals("ALL"))
      listAll(username);
     else
      try { listID(new Integer(richiesta),username); }
      catch (java.lang.NumberFormatException e)
       { sout.println("RESULT MANAGER: formato numerico non valido."); }

     o.println("RESULT MANAGER: richiesta completata.");
     sout.println(".end.");
   } catch (IOException e){}
     finally { try {s.close();} catch (IOException e) {}; }
  }

  /** Output di uno specifico risultato di un utente
   * @param ID ID del risultato desiderato
   * @param username username del richiedente
   */

  void listID(Integer ID, String username) {
   Object r=((Hashtable)agente.agentSystem.sharedObjects.get("Risultati")).get(ID);
   if (r==null) sout.println("RESULT MANAGER: ID del risultato non valido.");
   else mostraRisultato(r,username);
  }


  /** Output di tutti i risultati di un utente
   * @param username username del richiedente
   */

  void listAll(String username) {
   Enumeration e=((Hashtable)agente.agentSystem.sharedObjects.get("Risultati")).elements();
   while(e.hasMoreElements()) mostraRisultato(e.nextElement(),username);
  }


  /** Scrittura in output di uno specifico risultato
   * @param r oggetto risultato
   * @param username username del richiedente
   */

  void mostraRisultato(Object r, String username) {
   try {

    Method mio_getOggetto=r.getClass().getMethod("getOggetto",null);
    Method mio_getCosto=r.getClass().getMethod("getCosto",null);
    Method mio_getPlace=r.getClass().getMethod("getPlace",null);
    Method mio_finished=r.getClass().getMethod("finished",null);
    Method mio_getUsername=r.getClass().getMethod("getUsername",null);
/*
    if (!r.getUsername().equals(username)) return;

    if (r.getCosto()<0) {
     sout.print("L'oggetto <<"+r.getOggetto()+">> non si vende da nessuna parte :(");
    } else {
     sout.print("L'oggetto <<"+r.getOggetto()+">> costa "+r.getCosto()+" sul place "+r.getPlace());
    }
    if (r.finished()) sout.println("  : risultato definitivo");
    else sout.println("  : risultato parziale");
*/
    if (!((String)mio_getUsername.invoke(r,null)).equals(username)) return;

    if (((Double)mio_getCosto.invoke(r,null)).doubleValue()<0) {
     sout.print("L'oggetto <<"+((String)mio_getOggetto.invoke(r,null))+">> non si vende da nessuna parte :(");
    } else {
     sout.print("L'oggetto <<"+((String)mio_getOggetto.invoke(r,null))+">> costa "+((Double)mio_getCosto.invoke(r,null)).doubleValue()+" sul place "+ ((PlaceID)mio_getPlace.invoke(r,null)));
    }
    if (((Boolean)mio_finished.invoke(r,null)).booleanValue()) sout.println("  : risultato definitivo");
    else sout.println("  : risultato parziale");

   } catch (Exception e) { e.printStackTrace(); }
  }

}