
import SOMA.naming.*;

/**
 * Description:  Classe che si occupa di realizzare un'astrazione del risultato dei servizi
 * @author       Virgilio Pierini
 */

public class Risultato {

  protected int numAgenti;
  protected String oggetto,username;
  protected double costo;
  protected PlaceID place;

 /**
  * @param username nome dell'utente che ha richiesto il servizio
  * @param oggetto oggetto della ricerca
  * @param numAgenti numero di agenti al lavoro per la ricerca
  */

  public Risultato(String username, String oggetto, int numAgenti) {
   this.oggetto=oggetto;
   this.username=username;
   this.numAgenti=numAgenti;
   this.costo=-1;
   this.place=null;
  }


 /**
  * Utilizzato da un agente che comunichi di aver trovato un'offerta
  * (tiene conto che l'agente ha completato la sua esecuzione)
  * @param costo costo dell'oggetto
  * @param place place dove  in vendita l'oggetto
  */

  public synchronized void found(double costo, PlaceID place) {
   numAgenti--;
   if ((this.costo<0)||(costo<this.costo)) {
    this.costo=costo;
    this.place=place; }
  }


 /**
  * Utilizzato da un agente che comunichi di non aver trovato un'offerta
  * (tiene conto che l'agente ha completato la sua esecuzione)
  */

  public synchronized void notfound() { numAgenti--; }


 /**
  * Indica se il risultato pu essere considerato definitivo
  * @return true se non ci sono altri agenti da attendere
  */

  public boolean finished() { return (numAgenti==0); }


  public String getOggetto() { return oggetto; }
  public String getUsername() { return username; }
  public double getCosto() { return costo; }
  public PlaceID getPlace() { return place; }

  public String toString() {
   return "Per <<"+username+">> l'oggetto <<"+oggetto+">> costa "+costo+" su <<"+place+">> (ancora "+numAgenti+" agenti in giro)";
  }

}
